@@ -188,7 +188,7 @@ dbCellTileSrFunc(scx, fp)
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
 
     context.tc_scx = scx;
     context.tc_filter = fp;
@@ -307,7 +307,7 @@ dbCellUniqueTileSrFunc(scx, fp)
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
 
     context.tc_scx = scx;
     context.tc_filter = fp;
@@ -418,7 +418,7 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
 	return 0;
 
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
 
     filter.tf_func = func;
     filter.tf_arg = cdarg;
@@ -526,7 +526,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
     ASSERT(def != (CellDef *) NULL, "DBTreeSrLabels");
     if (!DBDescendSubcell(cellUse, xMask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
 
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
     {
@@ -592,7 +592,7 @@ dbCellLabelSrFunc(scx, fp)
     ASSERT(def != (CellDef *) NULL, "dbCellLabelSrFunc");
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
+	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
     
     if (fp->tf_tpath != (TerminalPath *) NULL)
     {
@@ -706,7 +706,7 @@ DBTreeSrCells(scx, xMask, func, cdarg)
     if (!DBDescendSubcell(cellUse, xMask))
 	return 0;
     if ((cellUse->cu_def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(cellUse->cu_def, (char *) NULL, TRUE))
+	if (!DBCellRead(cellUse->cu_def, (char *) NULL, TRUE, NULL))
 	    return 0;
 
     context.tc_scx = scx;
@@ -752,7 +752,7 @@ dbTreeCellSrFunc(scx, fp)
     else
     {
 	if ((use->cu_def->cd_flags & CDAVAILABLE) == 0)
-	    if (!DBCellRead(use->cu_def, (char *) NULL, TRUE))
+	    if (!DBCellRead(use->cu_def, (char *) NULL, TRUE, NULL))
 		return 0;
 	result = DBCellSrArea(scx, dbTreeCellSrFunc, (ClientData) fp);
     }
@@ -1004,7 +1004,7 @@ DBCellSrArea(scx, func, cdarg)
     context.tc_scx = scx;
 
     if ((scx->scx_use->cu_def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(scx->scx_use->cu_def, (char *) NULL, TRUE))
+	if (!DBCellRead(scx->scx_use->cu_def, (char *) NULL, TRUE, NULL))
 	    return 0;
     
     /* In order to make this work with zero-size areas, we first expand
@@ -1180,7 +1180,7 @@ DBCellEnum(cellDef, func, cdarg)
     filter.tf_func = func;
     filter.tf_arg = cdarg;
     if ((cellDef->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(cellDef, (char *) NULL, TRUE)) return 0;
+	if (!DBCellRead(cellDef, (char *) NULL, TRUE, NULL)) return 0;
     if (TiSrArea((Tile *) NULL, cellDef->cd_planes[PL_CELL],
 		&TiPlaneRect, dbEnumFunc, (ClientData) &filter))
 	return 1;
@@ -1732,14 +1732,28 @@ dbScaleCell(cellDef, scalen, scaled)
     /* Check consistency of several global pointers	*/
     /* WARNING:  This list may not be complete!		*/
 
-    /* Also scale the position of all labels */
+    /* Also scale the position of all labels. */
+    /* If labels are the rendered-font type, scale the size as well */
 
     if (cellDef->cd_labels)
     {
+	int i;
 	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
 	{
 	    DBScalePoint(&lab->lab_rect.r_ll, scalen, scaled);
 	    DBScalePoint(&lab->lab_rect.r_ur, scalen, scaled);
+
+	    if (lab->lab_font >= 0)
+	    {
+		DBScalePoint(&lab->lab_offset, scalen, scaled);
+		DBScaleValue(&lab->lab_size, scalen, scaled);
+
+		DBScalePoint(&lab->lab_bbox.r_ll, scalen, scaled);
+		DBScalePoint(&lab->lab_bbox.r_ur, scalen, scaled);
+		
+		for (i = 0; i < 4; i++)
+		    DBScalePoint(&lab->lab_corners[i], scalen, scaled);
+	    }
 	}
     }
 
