@@ -454,7 +454,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	    }
 	    else if (!strcmp(layername, "properties"))
 	    {
-		if (!dbReadProperties(cellDef, line, sizeof line, f)) goto badfile;
+		if (!dbReadProperties(cellDef, line, sizeof line, f, n, d)) goto badfile;
 		continue;
 	    }
 	    else if (!strcmp(layername, "end")) goto done;
@@ -1389,11 +1389,13 @@ nextLine:
  */
 
 bool
-dbReadProperties(cellDef, line, len, f)
+dbReadProperties(cellDef, line, len, f, scalen, scaled)
     CellDef *cellDef;	/* Cell whose elements are being read */
     char *line;		/* Line containing << elements >> */
     int len;		/* Size of buffer pointed to by line */
     FILE *f;		/* Input file */
+    int scalen;		/* Scale up by this factor */
+    int scaled;		/* Scale down by this factor */
 {
     char propertyname[128], propertyvalue[128], *storedvalue;
     int ntok;
@@ -1425,12 +1427,52 @@ dbReadProperties(cellDef, line, len, f)
 		TxError("Skipping bad property line: %s", line);
 		goto nextproperty;
 	    }
-	    storedvalue = StrDup((char **)NULL, propertyvalue);
-	    (void) DBPropPut(cellDef, propertyname, storedvalue);
 
 	    /* Go ahead and process the vendor GDS property */
 	    if (!strcmp(propertyname, "GDS_FILE"))
 		cellDef->cd_flags |= CDVENDORGDS;
+
+	    /* Also process FIXED_BBOX property, but do not keep	*/
+	    /* the property, as it should be regenerated on cell	*/
+	    /* output from the current scale.				*/
+
+	    if (!strcmp(propertyname, "FIXED_BBOX"))
+	    {
+		if (sscanf(propertyvalue, "%d %d %d %d",
+			&(cellDef->cd_bbox.r_xbot),
+			&(cellDef->cd_bbox.r_ybot),
+			&(cellDef->cd_bbox.r_xtop),
+			&(cellDef->cd_bbox.r_ytop)) != 4)
+		{
+		    TxError("Cannot read bounding box values in %s property",
+				propertyname);
+		    storedvalue = StrDup((char **)NULL, propertyvalue);
+		    (void) DBPropPut(cellDef, propertyname, storedvalue);
+		}
+		else
+		{
+		    if (scalen > 1)
+		    {
+			cellDef->cd_bbox.r_xbot *= scalen;
+			cellDef->cd_bbox.r_ybot *= scalen;
+			cellDef->cd_bbox.r_xtop *= scalen;
+			cellDef->cd_bbox.r_ytop *= scalen;
+		    }
+		    if (scaled > 1)
+		    {
+			cellDef->cd_bbox.r_xbot /= scaled;
+			cellDef->cd_bbox.r_ybot /= scaled;
+			cellDef->cd_bbox.r_xtop /= scaled;
+			cellDef->cd_bbox.r_ytop /= scaled;
+		    }
+		    cellDef->cd_flags |= CDFIXEDBBOX;
+		}
+	    }
+	    else
+	    {
+		storedvalue = StrDup((char **)NULL, propertyvalue);
+		(void) DBPropPut(cellDef, propertyname, storedvalue);
+	    }
 	}
 
 nextproperty:
@@ -2143,6 +2185,7 @@ DBCellWriteFile(cellDef, f)
     TileTypeBitMask typeMask, *sMask;
     int reducer;
     char *estring;
+    char lstring[256];
 
 #define FPRINTF(f,s)\
 {\
@@ -2227,8 +2270,6 @@ DBCellWriteFile(cellDef, f)
     /* Now labels */
     if (cellDef->cd_labels)
     {
-	char lstring[256];
-
 	FPRINTF(f, "<< labels >>\n");
 	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
 	{
@@ -2339,6 +2380,27 @@ DBCellWriteFile(cellDef, f)
 	DBPropEnum(cellDef, dbWritePropFunc, (ClientData)f);
     }
 
+    /* Fixed bounding box goes into a special property in output file	*/
+    /* This is not kept internally as a property, so that it can be	*/
+    /* read and written in the correct units without regard to internal	*/
+    /* changes in scaling.						*/
+
+    if (cellDef->cd_flags & CDFIXEDBBOX)
+    {
+	// If there were no explicit properties, then we need to
+	// write the header
+
+	if (cellDef->cd_props == (ClientData)NULL)
+	    FPRINTF(f, "<< properties >>\n");
+
+	sprintf(lstring, "string FIXED_BBOX %d %d %d %d\n",
+		cellDef->cd_bbox.r_xbot / reducer,
+		cellDef->cd_bbox.r_ybot / reducer,
+		cellDef->cd_bbox.r_xtop / reducer,
+		cellDef->cd_bbox.r_ytop / reducer);
+	FPRINTF(f, lstring);
+    }
+
     FPRINTF(f, "<< end >>\n");
 
     if (fflush(f) == EOF || ferror(f))
