@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtimestmp.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtimestmp.c,v 1.2 2009/05/30 03:13:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -137,7 +137,7 @@ DBFixMismatch()
 	mismatch = mismatch->mm_next;
 	if (cellDef->cd_flags & CDPROCESSED) continue;
 
-	(void) DBCellRead(cellDef, (char *) NULL, TRUE);
+	(void) DBCellRead(cellDef, (char *) NULL, TRUE, NULL);
 
 	/* Jimmy up the cell's current bounding box, so the following
 	 * procedure call will absolutely and positively know that
@@ -148,6 +148,7 @@ DBFixMismatch()
 	 */
 
 	cellDef->cd_bbox.r_xtop = cellDef->cd_bbox.r_xbot - 1;
+	cellDef->cd_extended.r_xtop = cellDef->cd_extended.r_xbot - 1;
 	DBReComputeBbox(cellDef);
 
 	/* Now, for each parent, recheck the parent in both the
