@@ -21,6 +21,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include <stdlib.h>		/* for atof() */
 #include <string.h>
 #include <ctype.h>
+#include <math.h>		/* for fabs() */
 
 #ifdef MAGIC_WRAPPER
 #include "tcltk/tclmagic.h"
@@ -43,6 +44,13 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/runstats.h"
 #include "ext2spice/ext2spice.h"
 
+// Structure passed to esHierVisit
+
+typedef struct _defflagsdata {
+   Def *def;
+   int flags;
+} DefFlagsData;
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -54,13 +62,15 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  */
 
 void
-ESGenerateHierarchy(inName)
+ESGenerateHierarchy(inName, flags)
     char *inName;
+    int flags;
 {
     int esHierVisit(), esMakePorts();	/* Forward declaration */
     Use u;
     Def *def;
     HierContext hc;
+    DefFlagsData dfd;
 
     u.use_def = efDefLook(inName);
     hc.hc_use = &u;
@@ -70,7 +80,9 @@ ESGenerateHierarchy(inName)
     EFHierSrDefs(&hc, esMakePorts, NULL);
     EFHierSrDefs(&hc, NULL, NULL);	/* Clear processed */
 
-    EFHierSrDefs(&hc, esHierVisit, (ClientData)(u.use_def));
+    dfd.def = u.use_def;
+    dfd.flags = flags; 
+    EFHierSrDefs(&hc, esHierVisit, (ClientData)(&dfd));
     EFHierSrDefs(&hc, NULL, NULL);	/* Clear processed */
 
     return;
@@ -105,6 +117,192 @@ GetHierNode(hc, name)
     return(nn->efnn_node);
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * spcHierWriteParams ---
+ *
+ * Write parameters to a device line in SPICE output.  This is normally
+ * restricted to subcircuit devices but may include other devices to
+ * accomodate various extensions to the basic SPICE format.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+spcHierWriteParams(hc, dev, scale, l, w, sdM)
+    HierContext *hc;
+    Dev *dev;           /* Dev being output */
+    float scale;        /* Scale transform for output */
+    int l;              /* Device length, in internal units */
+    int w;              /* Device width, in internal units */
+    float sdM;          /* Device multiplier */
+{
+    bool hierD;
+    DevParam *plist;
+    int parmval;
+    EFNode *dnode, *subnodeFlat = NULL;
+
+    /* Write all requested parameters to the subcircuit call.	*/
+
+    plist = efGetDeviceParams(EFDevTypes[dev->dev_type]);
+    while (plist != NULL)
+    {
+	switch (plist->parm_type[0])
+	{
+	    case 'a':
+		// Check for area of terminal node vs. device area
+		if (plist->parm_type[1] == '\0' || plist->parm_type[1] == '0')
+		{
+		    fprintf(esSpiceF, " %s=", plist->parm_name);
+		    parmval = dev->dev_area;
+		    if (esScale < 0)
+			fprintf(esSpiceF, "%g", parmval * scale * scale);
+		    else if (plist->parm_scale != 1.0)
+			fprintf(esSpiceF, "%g", parmval * scale * scale
+				* esScale * esScale * plist->parm_scale
+				* 1E-12);
+		    else
+			fprintf(esSpiceF, "%gp", parmval * scale * scale
+				* esScale * esScale);
+		}
+		else
+		{
+		    int pn;
+		    pn = plist->parm_type[1] - '0';
+		    if (pn >= dev->dev_nterm) pn = dev->dev_nterm - 1;
+
+		    dnode = GetHierNode(hc,
+			dev->dev_terms[pn].dterm_node->efnode_name->efnn_hier);
+
+		    // For parameter an followed by parameter pn,
+		    // process both at the same time
+
+		    if (plist->parm_next && plist->parm_next->parm_type[0] ==
+				'p' && plist->parm_next->parm_type[1] ==
+				plist->parm_type[1])
+		    {
+			spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+				scale, plist->parm_name, 
+				plist->parm_next->parm_name, sdM,
+				esSpiceF, w);
+			plist = plist->parm_next;
+		    }
+		    else
+		    {
+			spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+				scale, plist->parm_name, NULL, sdM,
+				esSpiceF, w);
+		    }
+		}
+
+		break;
+	    case 'p':
+		// Check for perimeter of terminal node vs. device perimeter
+		if (plist->parm_type[1] == '\0' || plist->parm_type[1] == '0')
+		{
+		    fprintf(esSpiceF, " %s=", plist->parm_name);
+		    parmval = dev->dev_perim;
+
+		    if (esScale < 0)
+			fprintf(esSpiceF, "%g", parmval * scale);
+		    else if (plist->parm_scale != 1.0)
+			fprintf(esSpiceF, "%g", parmval * scale
+				* esScale * plist->parm_scale * 1E-6);
+		    else
+			fprintf(esSpiceF, "%gu", parmval * scale * esScale);
+		}
+		else
+		{
+		    int pn;
+		    pn = plist->parm_type[1] - '0';
+		    if (pn >= dev->dev_nterm) pn = dev->dev_nterm - 1;
+
+		    dnode = GetHierNode(hc,
+			dev->dev_terms[pn].dterm_node->efnode_name->efnn_hier);
+
+		    // For parameter pn followed by parameter an,
+		    // process both at the same time
+
+		    if (plist->parm_next && plist->parm_next->parm_type[0] ==
+				'a' && plist->parm_next->parm_type[1] ==
+				plist->parm_type[1])
+		    {
+			spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+				scale, plist->parm_next->parm_name, 
+				plist->parm_name, sdM, esSpiceF, w);
+			plist = plist->parm_next;
+		    }
+		    else
+		    {
+			spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+				scale, NULL, plist->parm_name, sdM,
+				esSpiceF, w);
+		    }
+		}
+
+		break;
+	    case 'l':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		if (esScale < 0)
+		    fprintf(esSpiceF, "%g", l * scale);
+		else if (plist->parm_scale != 1.0)
+		    fprintf(esSpiceF, "%g", l * scale * esScale
+				* plist->parm_scale * 1E-6);
+		else
+		    fprintf(esSpiceF, "%gu", l * scale * esScale);
+		break;
+	    case 'w':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		if (esScale < 0)
+		    fprintf(esSpiceF, "%g", w * scale);
+		else if (plist->parm_scale != 1.0)
+		    fprintf(esSpiceF, "%g", w * scale * esScale
+				* plist->parm_scale * 1E-6);
+		else
+		    fprintf(esSpiceF, "%gu", w * scale * esScale);
+		break;
+	    case 's':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		subnodeFlat = spcdevSubstrate(hc->hc_hierName,
+			dev->dev_subsnode->efnode_name->efnn_hier,
+			dev->dev_type, esSpiceF);
+		break;
+	    case 'x':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		if (esScale < 0)
+		    fprintf(esSpiceF, "%g", dev->dev_rect.r_xbot * scale);
+		else if (plist->parm_scale != 1.0)
+		    fprintf(esSpiceF, "%g", dev->dev_rect.r_xbot * scale
+				* esScale * plist->parm_scale * 1E-6);
+		else
+		    fprintf(esSpiceF, "%gu", dev->dev_rect.r_xbot * scale
+				* esScale);
+		break;
+	    case 'y':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		if (esScale < 0)
+		    fprintf(esSpiceF, "%g", dev->dev_rect.r_ybot * scale);
+		else if (plist->parm_scale != 1.0)
+		    fprintf(esSpiceF, "%g", dev->dev_rect.r_ybot * scale
+				* esScale * plist->parm_scale * 1E-6);
+		else
+		    fprintf(esSpiceF, "%gu", dev->dev_rect.r_ybot * scale
+				* esScale);
+		break;
+	    case 'r':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		fprintf(esSpiceF, "%f", (double)(dev->dev_res));
+		break;
+	    case 'c':
+		fprintf(esSpiceF, " %s=", plist->parm_name);
+		fprintf(esSpiceF, "%ff", (double)(dev->dev_cap));
+		break;
+	}
+	plist = plist->parm_next;
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -160,6 +358,7 @@ esOutputHierResistor(hc, dev, scale, term1, term2, has_model, l, w, dscale)
     {
 	fprintf(esSpiceF, " %f", ((double)(dev->dev_res)
 			/ (double)(dscale)) / (double)sdM);
+	spcHierWriteParams(hc, dev, scale, l, w, sdM);
     }
     else
     {
@@ -176,6 +375,7 @@ esOutputHierResistor(hc, dev, scale, term1, term2, has_model, l, w, dscale)
 		(float)w * scale * esScale,
 		(float)((l * scale * esScale) / dscale));
 	}
+	spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	if (sdM != 1.0)
 	    fprintf(esSpiceF, " M=%g", sdM);
     }
@@ -303,6 +503,7 @@ spcdevHierVisit(hc, dev, scale)
     {
 	case DEV_RES:
 	case DEV_CAP:
+	case DEV_CAPREV:
 	    if (dev->dev_type == esNoModelType)
 		has_model = FALSE;
 	    break;
@@ -336,12 +537,15 @@ spcdevHierVisit(hc, dev, scale)
 	    devchar = 'Q';
 	    break;
 	case DEV_DIODE:
+	case DEV_NDIODE:
+	case DEV_PDIODE:
 	    devchar = 'D';
 	    break;
 	case DEV_RES:
 	    devchar = 'R';
 	    break;
 	case DEV_CAP:
+	case DEV_CAPREV:
 	    devchar = 'C';
 	    break;
 	case DEV_SUBCKT:
@@ -368,9 +572,12 @@ spcdevHierVisit(hc, dev, scale)
 		if (esDoResistorTee) fprintf(esSpiceF, "A");
 		break;
 	    case DEV_DIODE:
+	    case DEV_NDIODE:
+	    case DEV_PDIODE:
 		fprintf(esSpiceF, "%d", esDiodeNum++);
 		break;
 	    case DEV_CAP:
+	    case DEV_CAPREV:
 		fprintf(esSpiceF, "%d", esCapNum++);
 		break;
 	    case DEV_SUBCKT:
@@ -410,6 +617,7 @@ spcdevHierVisit(hc, dev, scale)
 			"base", esSpiceF);
 
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    break;
 
 	case DEV_MSUBCKT:
@@ -477,163 +685,8 @@ spcdevHierVisit(hc, dev, scale)
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 
 	    /* Write all requested parameters to the subcircuit call.	*/
-
 	    sdM = getCurDevMult();
-	    while (plist != NULL)
-	    {
-		switch (plist->parm_type[0])
-		{
-		    case 'a':
-			// Check for area of terminal node vs. device area
-			if (plist->parm_type[1] == '\0' || plist->parm_type[1] == '0')
-			{
-			    fprintf(esSpiceF, " %s=", plist->parm_name);
-			    parmval = dev->dev_area;
-			    if (esScale < 0)
-				fprintf(esSpiceF, "%g", parmval * scale * scale);
-			    else if (plist->parm_scale != 1.0)
-				fprintf(esSpiceF, "%g", parmval * scale * scale
-					* esScale * esScale * plist->parm_scale
-					* 1E-12);
-			    else
-				fprintf(esSpiceF, "%gp", parmval * scale * scale
-					* esScale * esScale);
-			}
-			else
-			{
-			    int pn;
-			    pn = plist->parm_type[1] - '0';
-			    if (pn >= dev->dev_nterm) pn = dev->dev_nterm - 1;
-
-			    dnode = GetHierNode(hc,
-				dev->dev_terms[pn].dterm_node->efnode_name->efnn_hier);
-
-			    // For parameter an followed by parameter pn,
-			    // process both at the same time
-
-			    if (plist->parm_next && plist->parm_next->parm_type[0] ==
-					'p' && plist->parm_next->parm_type[1] ==
-					plist->parm_type[1])
-			    {
-				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
-					scale, plist->parm_name, 
-					plist->parm_next->parm_name, sdM,
-					esSpiceF, w);
-				plist = plist->parm_next;
-			    }
-			    else
-			    {
-				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
-					scale, plist->parm_name, NULL, sdM,
-					esSpiceF, w);
-			    }
-			}
-	
-			break;
-		    case 'p':
-			// Check for perimeter of terminal node vs. device perimeter
-			if (plist->parm_type[1] == '\0' || plist->parm_type[1] == '0')
-			{
-			    fprintf(esSpiceF, " %s=", plist->parm_name);
-			    parmval = dev->dev_perim;
-
-			    if (esScale < 0)
-				fprintf(esSpiceF, "%g", parmval * scale);
-			    else if (plist->parm_scale != 1.0)
-				fprintf(esSpiceF, "%g", parmval * scale
-					* esScale * plist->parm_scale * 1E-6);
-			    else
-				fprintf(esSpiceF, "%gu", parmval * scale * esScale);
-			}
-			else
-			{
-			    int pn;
-			    pn = plist->parm_type[1] - '0';
-			    if (pn >= dev->dev_nterm) pn = dev->dev_nterm - 1;
-
-			    dnode = GetHierNode(hc,
-				dev->dev_terms[pn].dterm_node->efnode_name->efnn_hier);
-
-			    // For parameter pn followed by parameter an,
-			    // process both at the same time
-
-			    if (plist->parm_next && plist->parm_next->parm_type[0] ==
-					'a' && plist->parm_next->parm_type[1] ==
-					plist->parm_type[1])
-			    {
-				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
-					scale, plist->parm_next->parm_name, 
-					plist->parm_name, sdM, esSpiceF, w);
-				plist = plist->parm_next;
-			    }
-			    else
-			    {
-				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
-					scale, NULL, plist->parm_name, sdM,
-					esSpiceF, w);
-			    }
-			}
-	
-			break;
-		    case 'l':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			if (esScale < 0)
-			    fprintf(esSpiceF, "%g", l * scale);
-			else if (plist->parm_scale != 1.0)
-			    fprintf(esSpiceF, "%g", l * scale * esScale
-					* plist->parm_scale * 1E-6);
-			else
-			    fprintf(esSpiceF, "%gu", l * scale * esScale);
-			break;
-		    case 'w':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			if (esScale < 0)
-			    fprintf(esSpiceF, "%g", w * scale);
-			else if (plist->parm_scale != 1.0)
-			    fprintf(esSpiceF, "%g", w * scale * esScale
-					* plist->parm_scale * 1E-6);
-			else
-			    fprintf(esSpiceF, "%gu", w * scale * esScale);
-			break;
-		    case 's':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			subnodeFlat = spcdevSubstrate(hc->hc_hierName,
-				subnode->efnode_name->efnn_hier,
-				dev->dev_type, esSpiceF);
-			break;
-		    case 'x':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			if (esScale < 0)
-			    fprintf(esSpiceF, "%g", dev->dev_rect.r_xbot * scale);
-			else if (plist->parm_scale != 1.0)
-			    fprintf(esSpiceF, "%g", dev->dev_rect.r_xbot * scale
-					* esScale * plist->parm_scale * 1E-6);
-			else
-			    fprintf(esSpiceF, "%gu", dev->dev_rect.r_xbot * scale
-					* esScale);
-			break;
-		    case 'y':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			if (esScale < 0)
-			    fprintf(esSpiceF, "%g", dev->dev_rect.r_ybot * scale);
-			else if (plist->parm_scale != 1.0)
-			    fprintf(esSpiceF, "%g", dev->dev_rect.r_ybot * scale
-					* esScale * plist->parm_scale * 1E-6);
-			else
-			    fprintf(esSpiceF, "%gu", dev->dev_rect.r_ybot * scale
-					* esScale);
-			break;
-		    case 'r':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			fprintf(esSpiceF, "%f", (double)(dev->dev_res));
-			break;
-		    case 'c':
-			fprintf(esSpiceF, " %s=", plist->parm_name);
-			fprintf(esSpiceF, "%ff", (double)(dev->dev_cap));
-			break;
-		}
-		plist = plist->parm_next;
-	    }
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    if (sdM != 1.0)
 		fprintf(esSpiceF, " M=%g", sdM);
 	    break;
@@ -673,6 +726,7 @@ spcdevHierVisit(hc, dev, scale)
 	    break;
 
 	case DEV_DIODE:
+	case DEV_PDIODE:
 	    if (source == NULL) break;
 
 	    /* Diode is "Dnnn top bottom model"	*/
@@ -680,10 +734,36 @@ spcdevHierVisit(hc, dev, scale)
 	    spcdevOutNode(hc->hc_hierName,
 			gate->dterm_node->efnode_name->efnn_hier,
 			"diode_top", esSpiceF);
-	    spcdevOutNode(hc->hc_hierName,
+	    if (dev->dev_nterm > 1)
+		spcdevOutNode(hc->hc_hierName,
 			source->dterm_node->efnode_name->efnn_hier,
 			"diode_bot", esSpiceF);
+	    else if (subnode)
+		spcdevOutNode(hc->hc_hierName,
+			subnode->efnode_name->efnn_hier,
+			"diode_bot", esSpiceF);
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    break;
+
+	case DEV_NDIODE:
+	    if (source == NULL) break;
+
+	    /* Diode is "Dnnn bottom top model"	*/
+
+	    if (dev->dev_nterm > 1)
+		spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"diode_bot", esSpiceF);
+	    else if (subnode)
+		spcdevOutNode(hc->hc_hierName,
+			subnode->efnode_name->efnn_hier,
+			"diode_bot", esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			gate->dterm_node->efnode_name->efnn_hier,
+			"diode_top", esSpiceF);
+	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    break;
 
 	case DEV_CAP:
@@ -711,6 +791,7 @@ spcdevHierVisit(hc, dev, scale)
 	    {
 		fprintf(esSpiceF, " %ffF", (double)sdM *
 				(double)(dev->dev_cap));
+		spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    }
 	    else
 	    {
@@ -726,6 +807,54 @@ spcdevHierVisit(hc, dev, scale)
 			w * scale * esScale,
 			l * scale * esScale);
 		}
+		spcHierWriteParams(hc, dev, scale, l, w, sdM);
+		if (sdM != 1.0)
+		    fprintf(esSpiceF, " M=%g", sdM);
+	    }
+	    break;
+
+	case DEV_CAPREV:
+	    if (source == NULL) break;
+
+	    /* Capacitor is "Cnnn bottom top value"	*/
+	    /* extraction sets top=source bottom=gate	*/
+	    /* extracted units are fF; output is in fF */
+
+	    spcdevOutNode(hc->hc_hierName,
+			gate->dterm_node->efnode_name->efnn_hier,
+			"cap_bot", esSpiceF);
+	    spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"cap_top", esSpiceF);
+
+	    sdM = getCurDevMult();
+
+	    /* SPICE has two capacitor types.  If the "name" (EFDevTypes) is */
+	    /* "None", the simple capacitor type is used, and a value given. */
+	    /* If not, the "semiconductor capacitor" is used, and L and W    */
+	    /* and the device name are output.				     */
+
+	    if (!has_model)
+	    {
+		fprintf(esSpiceF, " %ffF", (double)sdM *
+				(double)(dev->dev_cap));
+		spcHierWriteParams(hc, dev, scale, l, w, sdM);
+	    }
+	    else
+	    {
+		fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
+
+		if (esScale < 0)
+		{
+		    fprintf(esSpiceF, " w=%g l=%g", w*scale, l*scale);
+		}
+		else
+		{
+		    fprintf(esSpiceF, " w=%gu l=%gu",
+			w * scale * esScale,
+			l * scale * esScale);
+		}
+		spcHierWriteParams(hc, dev, scale, l, w, sdM);
 		if (sdM != 1.0)
 		    fprintf(esSpiceF, " M=%g", sdM);
 	    }
@@ -768,6 +897,7 @@ spcdevHierVisit(hc, dev, scale)
 			w * scale * esScale,
 			l * scale * esScale);
 	    }
+	    spcHierWriteParams(hc, dev, scale, l, w, sdM);
 	    if (sdM != 1.0)
 		fprintf(esSpiceF, " M=%g", sdM);
 
@@ -884,6 +1014,7 @@ spcdevHierMergeVisit(hc, dev, scale)
 			m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
 		    break;
 		case DEV_CAP:
+		case DEV_CAPREV:
 		    if (fp->dev->dev_type == esNoModelType)
 			m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
 				/ cfp->dev->dev_cap);
@@ -940,7 +1071,7 @@ spccapHierVisit(hc, hierName1, hierName2, cap)
     double cap;
 {
     cap = cap / 1000;
-    if (cap <= EFCapThreshold)
+    if (fabs(cap) <= EFCapThreshold)
 	return 0;
 
     fprintf(esSpiceF, esSpiceCapFormat, esCapNum++,
@@ -1006,12 +1137,12 @@ spcresistHierVisit(hc, hierName1, hierName2, res)
  */
 
 int
-spcsubHierVisit(hc, node, res, cap, resstr)
+spcsubHierVisit(hc, node, res, cap, resstrptr)
     HierContext *hc;
     EFNode *node;
     int res; 		// Unused
     double cap;		// Unused
-    char **resstr;
+    char **resstrptr;
 {
     HierName *hierName;
     char *nsn;
@@ -1020,7 +1151,7 @@ spcsubHierVisit(hc, node, res, cap, resstr)
     {
 	hierName = (HierName *) node->efnode_name->efnn_hier;
 	nsn = nodeSpiceHierName(hc, hierName);
-	*resstr = StrDup((char **)NULL, nsn);
+	*resstrptr = StrDup((char **)NULL, nsn);
 	return 1;
     }
     return 0;
@@ -1079,7 +1210,7 @@ spcnodeHierVisit(hc, node, res, cap)
 	fprintf(esSpiceF, "** %s == %s\n", ntmp, nsn);
     }
     cap = cap  / 1000;
-    if (cap > EFCapThreshold)
+    if (fabs(cap) > EFCapThreshold)
     {
 	fprintf(esSpiceF, esSpiceCapFormat, esCapNum++, nsn, cap,
 			  (isConnected) ?  "\n" : " **FLOATING\n");
@@ -1126,7 +1257,8 @@ char *nodeSpiceHierName(hc, hname)
     EFNode *node;
     Def *def = hc->hc_use->use_def;
 
-    he = HashLookOnly(&def->def_nodes, EFHNToStr(hname));
+    // he = HashLookOnly(&def->def_nodes, EFHNToStr(hname));
+    he = HashLookOnly(&efNodeHashTable, (char *)hname);
     if (he == NULL) return "error";
 
     nn = (EFNodeName *) HashGetValue(he);
@@ -1278,6 +1410,7 @@ mergeThem:
 		        m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
 		    break;
 		case DEV_CAP:
+		case DEV_CAPREV:
 		    if (fp->dev->dev_type == esNoModelType)
 		        m = esFMult[cfp->esFMIndex] + (fp->dev->dev_cap
 				/ cfp->dev->dev_cap);
@@ -1442,6 +1575,87 @@ esMakePorts(hc, cdata)
 	    //		name, portname);
 	}
     }
+
+    // Now do the same thing for parasitic connections into subcells
+    // However, restrict the number of ports based on "cthresh".
+
+    for (conn = (Connection *)def->def_caps; conn; conn = conn->conn_next)
+    {
+	for (j = 0; j < 2; j++)
+	{
+	    name = (j == 0) ? conn->conn_1.cn_name : conn->conn_2.cn_name;
+	    locname = (j == 0) ? conn->conn_2.cn_name : conn->conn_1.cn_name;
+	    if ((tptr = strchr(name, '/')) == NULL)
+		continue;
+
+	    // Ignore capacitances that are less than the threshold.
+	    // In particular, this keeps parasitics out of the netlist for
+	    // LVS purposes if "cthresh" is set to "infinite".
+
+	    if (fabs((double)conn->conn_cap) < EFCapThreshold) continue;
+
+	    portname = name;
+	    updef = def;
+
+	    while (tptr != NULL)
+	    {
+		/* Ignore array information for the purpose of tracing	*/	
+		/* the cell definition hierarchy.			*/
+
+		aptr = strchr(portname, '[');
+		if ((aptr == NULL) || (aptr > tptr))
+		    *tptr = '\0';
+		else
+		    *aptr = '\0';
+
+		// Find the cell for the instance
+		portdef = NULL;
+		for (use = updef->def_uses; use; use = use->use_next)
+		{
+		    if (!strcmp(use->use_id, portname))
+		    {
+			portdef = use->use_def;
+			break;
+		    }
+		}
+		if ((aptr == NULL) || (aptr > tptr))
+		    *tptr = '/';
+		else
+		    *aptr = '[';
+		portname = tptr + 1;
+
+		// Find the net of portname in the subcell and
+		// make it a port if it is not already.
+
+		if (portdef)
+		{
+		    he = HashFind(&portdef->def_nodes, portname);
+		    nn = (EFNodeName *) HashGetValue(he);
+		    if (nn == NULL)
+		    {
+			efBuildNode(portdef, FALSE, portname, 0.0,
+					0, 0, NULL, NULL, 0);
+			nn = (EFNodeName *) HashGetValue(he);
+		    }
+
+		    if (!(nn->efnn_node->efnode_flags & EF_PORT))
+		    {
+			nn->efnn_node->efnode_flags |= EF_PORT;
+			nn->efnn_port = -1;	// Will be sorted later
+		    }
+		}
+
+		if ((tptr = strchr(portname, '/')) == NULL)
+		    break;
+		if (portdef == NULL) break;	// Error condition?
+
+		updef = portdef;
+	    }
+	    // Diagnostic
+	    // TxPrintf("Connection in %s to net %s (%s)\n", def->def_name,
+	    //		name, portname);
+	}
+    }
     return 0;
 }
 
@@ -1465,9 +1679,15 @@ esHierVisit(hc, cdata)
 {
     HierContext *hcf;
     Def *def = hc->hc_use->use_def;
-    Def *topdef = (Def *)cdata;
+    Def *topdef;
     EFNode *snode;
     char *resstr = NULL;
+    DefFlagsData *dfd;
+    int flags;
+
+    dfd = (DefFlagsData *)cdata;
+    topdef = dfd->def;
+    flags = dfd->flags;
 
     /* Cells without any contents (devices or subcircuits) will	*/
     /* be absorbed into their parents.  Use this opportunity to	*/
@@ -1486,7 +1706,7 @@ esHierVisit(hc, cdata)
     } 
 
     /* Flatten this definition only */
-    hcf = EFFlatBuildOneLevel(hc->hc_use->use_def);
+    hcf = EFFlatBuildOneLevel(hc->hc_use->use_def, flags);
 
     /* If definition has been marked as having no devices, then this	*/
     /* def is not to be output unless it is the top level.		*/
@@ -1498,8 +1718,8 @@ esHierVisit(hc, cdata)
     }
 
     /* Generate subcircuit header */
-    if (def != topdef)
-	topVisit(hc->hc_use->use_def);
+    if ((def != topdef) || (def->def_flags & DEF_SUBCIRCUIT))
+	topVisit(def);
     else
 	fprintf(esSpiceF, "\n* Top level circuit %s\n\n", topdef->def_name);
 
@@ -1529,17 +1749,20 @@ esHierVisit(hc, cdata)
     sprintf( esSpiceCapFormat,  "C%%d %%s %%s %%.%dlffF\n", esCapAccuracy);
     EFHierVisitCaps(hcf, spccapHierVisit, (ClientData)NULL);
 
-    /* Find the substrate node */
-    EFHierVisitNodes(hcf, spcsubHierVisit, (ClientData)&resstr);
-    if (resstr == NULL) resstr = StrDup((char **)NULL, "0");
+    if (EFCompat == FALSE)
+    {
+	/* Find the substrate node */
+	EFHierVisitNodes(hcf, spcsubHierVisit, (ClientData)&resstr);
+	if (resstr == NULL) resstr = StrDup((char **)NULL, "0");
 
-    /* Output lumped capacitance and resistance to substrate */
-    sprintf( esSpiceCapFormat,  "C%%d %%s %s %%.%dlffF%%s\n",
+	/* Output lumped capacitance and resistance to substrate */
+	sprintf( esSpiceCapFormat,  "C%%d %%s %s %%.%dlffF%%s\n",
 			resstr, esCapAccuracy);
-    EFHierVisitNodes(hcf, spcnodeHierVisit, (ClientData) NULL);
-    freeMagic(resstr);
+	EFHierVisitNodes(hcf, spcnodeHierVisit, (ClientData) NULL);
+	freeMagic(resstr);
+    }
 
-    if (def != topdef)
+    if ((def != topdef) || (def->def_flags & DEF_SUBCIRCUIT))
 	fprintf(esSpiceF, ".ends\n\n");
     else
 	fprintf(esSpiceF, ".end\n\n");
