@@ -203,6 +203,9 @@ efBuildNode(def, isSubsnode, nodeName, nodeCap, x, y, layerName, av, ac)
     newnode->efnode_prev = (EFNodeHdr *) &def->def_firstn;
     def->def_firstn.efnode_next->efnhdr_prev = (EFNodeHdr *) newnode;
     def->def_firstn.efnode_next = (EFNodeHdr *) newnode;
+
+    /* If isSubsnode was TRUE, then turn off backwards compatibility mode */
+    if (isSubsnode == TRUE) EFCompat = FALSE;
 }
 
 /*
@@ -514,7 +517,18 @@ efBuildDeviceParams(name, argc, argv)
 	else
 	    newparm->parm_scale = 1.0;
 
-	newparm->parm_name = StrDup((char **)NULL, pptr + 1);
+	// For parameters defined for cell defs, copy the whole
+	// expression verbatim into parm_name.  parm_type is
+	// reassigned to be a numerical order.
+
+	if (name[0] == ':')
+	{
+	    newparm->parm_name = StrDup((char **)NULL, argv[n]);
+	    newparm->parm_type[1] = '0' + n / 10;
+	    newparm->parm_type[0] = '0' + n % 10;
+	}
+	else
+	    newparm->parm_name = StrDup((char **)NULL, pptr + 1);
 	newparm->parm_next = plist;
 	plist = newparm;
     }
@@ -581,10 +595,13 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    argstart = 3;
 	    break;
 	case DEV_DIODE:
+	case DEV_NDIODE:
+	case DEV_PDIODE:
 	    argstart = 0;
 	    break;
 	case DEV_RES:
 	case DEV_CAP:
+	case DEV_CAPREV:
 	    if (hasModel)
 		argstart = 2;
 	    break;
@@ -592,51 +609,50 @@ efBuildDevice(def, class, type, r, argc, argv)
 	case DEV_MSUBCKT:
 	case DEV_RSUBCKT:
 	    argstart = 0;
+    }
 
-	    /* Parse initial arguments for parameters */
-	    while ((pptr = strchr(argv[argstart], '=')) != NULL)
-	    {
-		pptr++;
-		switch(*argv[argstart])
+    /* Parse initial arguments for parameters */
+    while ((pptr = strchr(argv[argstart], '=')) != NULL)
+    {
+	pptr++;
+	switch(*argv[argstart])
+	{
+	    case 'a':
+		if ((pptr - argv[argstart]) == 2)
+		    devtmp.dev_area = atoi(pptr);
+		else
 		{
-		    case 'a':
-			if ((pptr - argv[argstart]) == 2)
-			    devtmp.dev_area = atoi(pptr);
-			else
-			{
-			    pn = *(argv[argstart] + 1) - '0';
-			    if (pn == 0)
-				devtmp.dev_area = atoi(pptr);
-			    /* Otherwise, punt */
-			}
-			break;
-		    case 'p':
-			if ((pptr - argv[argstart]) == 2)
-			    devtmp.dev_perim = atoi(pptr);
-			else
-			{
-			    pn = *(argv[argstart] + 1) - '0';
-			    if (pn == 0)
-				devtmp.dev_perim = atoi(pptr);
-			    /* Otherwise, punt */
-			}
-			break;
-		    case 'l':
-			devtmp.dev_length = atoi(pptr);
-			break;
-		    case 'w':
-			devtmp.dev_width = atoi(pptr);
-			break;
-		    case 'c':
-			devtmp.dev_cap = (float)atof(pptr);
-			break;
-		    case 'r':
-			devtmp.dev_res = (float)atof(pptr);
-			break;
+		    pn = *(argv[argstart] + 1) - '0';
+		    if (pn == 0)
+			devtmp.dev_area = atoi(pptr);
+		    /* Otherwise, punt */
 		}
-		argstart++;
-	    }
-	    break;
+		break;
+	    case 'p':
+		if ((pptr - argv[argstart]) == 2)
+		    devtmp.dev_perim = atoi(pptr);
+		else
+		{
+		    pn = *(argv[argstart] + 1) - '0';
+		    if (pn == 0)
+			devtmp.dev_perim = atoi(pptr);
+		    /* Otherwise, punt */
+		}
+		break;
+	    case 'l':
+		devtmp.dev_length = atoi(pptr);
+		break;
+	    case 'w':
+		devtmp.dev_width = atoi(pptr);
+		break;
+	    case 'c':
+		devtmp.dev_cap = (float)atof(pptr);
+		break;
+	    case 'r':
+		devtmp.dev_res = (float)atof(pptr);
+		break;
+	}
+	argstart++;
     }
 
     /* Check for optional substrate node */
@@ -645,10 +661,13 @@ efBuildDevice(def, class, type, r, argc, argv)
     {
 	case DEV_RES:
 	case DEV_CAP:
+	case DEV_CAPREV:
 	case DEV_RSUBCKT:
 	case DEV_MSUBCKT:
 	case DEV_SUBCKT:
 	case DEV_DIODE:
+	case DEV_NDIODE:
+	case DEV_PDIODE:
 	    n = argc - argstart;
 	    if ((n % 3) == 1)
 	    {
@@ -728,6 +747,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 
 	    break;
 	case DEV_CAP:
+	case DEV_CAPREV:
 	    if (hasModel && StrIsInt(argv[0]) && StrIsInt(argv[1]))
 	    {
 		newdev->dev_length = atoi(argv[0]);
