@@ -43,7 +43,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #ifndef MAGIC_WRAPPER
 /* This must match the definition for extDevTable in extract/ExtBasic.c */
 char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
-		"devcap", "diode", "subckt", "rsubckt", "msubckt", NULL};
+		"devcap", "devcaprev", "diode", "pdiode", "ndiode",
+		"subckt", "rsubckt", "msubckt", NULL};
 #endif
 
 /*
@@ -136,7 +137,7 @@ EFReadFile(name, dosubckt, resist, noscale)
     if (def == NULL)
 	def = efDefNew(name);
 
-    rc = efReadDef(def, dosubckt, resist, noscale);
+    rc = efReadDef(def, dosubckt, resist, noscale, TRUE);
     if (EFArgTech) EFTech = StrDup((char **) NULL, EFArgTech);
     if (EFScale == 0.0) EFScale = 1.0;
 
@@ -164,9 +165,9 @@ EFReadFile(name, dosubckt, resist, noscale)
  */
 
 bool
-efReadDef(def, dosubckt, resist, noscale)
+efReadDef(def, dosubckt, resist, noscale, toplevel)
    Def *def;
-   bool dosubckt, resist, noscale;
+   bool dosubckt, resist, noscale, toplevel;
 {
     int argc, ac, n;
     EFCapValue cap;
@@ -290,9 +291,12 @@ readfile:
 			ac = 10;
 			break;
 		    case DEV_DIODE:
+		    case DEV_NDIODE:
+		    case DEV_PDIODE:
 			ac = 7;
 			break;
 		    case DEV_CAP:
+		    case DEV_CAPREV:
 		    case DEV_RES:
 			if (!strcmp(argv[2], "None"))	/* Has device value */
 			   ac = 8;
@@ -558,13 +562,14 @@ resistChanged:
     /* If we are considering standard cells, only the first level of	*/
     /* subcircuits is meaningful.					*/
 
-    if (def->def_flags & DEF_SUBCIRCUIT)
+    if ((def->def_flags & DEF_SUBCIRCUIT) && (toplevel != TRUE))
 	DoSubCircuit = FALSE;
 
     /* Read in each def that has not yet been read in */
     for (use = def->def_uses; use; use = use->use_next)
 	if ((use->use_def->def_flags & DEF_AVAILABLE) == 0)
-	    if(efReadDef(use->use_def, DoSubCircuit, resist, noscale) != TRUE)
+	    if (efReadDef(use->use_def, DoSubCircuit, resist, noscale, FALSE)
+			!= TRUE)
 		rc = FALSE;
 
     return rc;
