@@ -50,7 +50,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  * been marked with their associated region.
  */
 ClientData extUnInit = (ClientData) CLIENTDEFAULT;
-NodeRegion *save_subsnode = (NodeRegion *)NULL;
 
 
 /* ------------------------ Data local to this file ------------------- */
@@ -163,7 +162,7 @@ extFileOpen(def, file, mode, prealfile)
 			 * a string holding the name of the .ext file.
 			 */
 {
-    char namebuf[512], *name, *endp;
+    char namebuf[512], *name, *endp, *ends;
     int len;
     FILE *rfile;
 
@@ -171,7 +170,9 @@ extFileOpen(def, file, mode, prealfile)
     else if (def->cd_file)
     {
 	name = def->cd_file;
-	if (endp = strrchr(def->cd_file, '.'))
+	ends = strrchr(def->cd_file, '/');
+	if (ends == NULL) ends = def->cd_file;
+	if (endp = strrchr(ends + 1, '.'))
 	{
 	    name = namebuf;
 	    len = endp - def->cd_file;
@@ -232,7 +233,6 @@ extCellFile(def, f, doLength)
     /* Extract the mask information in this cell */
     reg = (NodeRegion *) NULL;
     if (!SigInterruptPending) reg = extBasic(def, f);
-    save_subsnode = glob_subsnode;
 
     /* Do hierarchical extraction */
     extParentUse->cu_def = def;
@@ -279,6 +279,8 @@ extHeader(def, f)
     FILE *f;		/* Write to this file */
 {
     int n;
+    bool propfound;
+    char *propvalue;
 
     /* Output a timestamp (should be first) */
     fprintf(f, "timestamp %d\n", def->cd_timestamp);
@@ -310,6 +312,19 @@ extHeader(def, f)
 	fprintf(f, " %d", ExtCurStyle->exts_resistByResistClass[n]);
     fprintf(f, "\n");
 
+    /* Output any parameters defined for this cell that	*/
+    /* are to be passed to instances of the cell	*/
+    /* (created by defining property "parameter")	*/
+
+    propvalue = (char *)DBPropGet(def, "parameter", &propfound);
+    if (propfound)
+    {
+	// Use device parameter table to store the cell def parameters,
+	// but preface name with ":" to avoid any conflict with device
+	// names.
+	fprintf(f, "parameters :%s %s\n", def->cd_name, propvalue);
+    }
+
     /* Output all calls on subcells */
     (void) DBCellEnum(def, extOutputUsesFunc, (ClientData) f);
 }
