@@ -572,6 +572,7 @@ extTechStyleInit(style)
 
     style->exts_sidePlanes = style->exts_overlapPlanes = 0;
     TTMaskZero(&style->exts_transMask);
+    style->exts_activeTypes = DBAllButSpaceAndDRCBits;
 
     for (r = 0; r < NP; r++)
     {
@@ -2450,7 +2451,24 @@ ExtTechLine(sectionName, argc, argv)
 	case RESIST: {
 	    float chop = 1.0;
 
-	    val = atoi(argv[2]);
+	    if (!StrIsInt(argv[2]))
+	    {
+		if (!strcmp(argv[2], "None"))
+		{
+		    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+			if (TTMaskHasType(&types1, t))
+			    TTMaskClearType(&ExtCurStyle->exts_activeTypes, t);
+		    break;
+		}
+		else
+		{
+		    TxError("Resist argument must be integer or \"None\".\n");
+		    break;
+		}
+	    }
+	    else
+		val = atoi(argv[2]);
+
 	    if (argc == 4)
 		chop = atof(argv[3]);
 	    class = ExtCurStyle->exts_numResistClasses++;
@@ -2567,7 +2585,7 @@ extTechFinalStyle(style)
 {
     TileTypeBitMask maskBits;
     TileType r, s, t;
-    int p1, missing, conflict;
+    int p, p1, missing, conflict;
     int indicis[NP];
 
     for (r = TT_TECHDEPBASE; r < DBNumTypes; r++)
@@ -2612,7 +2630,6 @@ extTechFinalStyle(style)
     {
 	TileTypeBitMask rmask;
 	PlaneMask pMask;
-	int p;
 	TileType q;
 
 	if (!DBIsContact(s)) continue;
@@ -2681,6 +2698,31 @@ extTechFinalStyle(style)
 	    }
 	}
 
+    /* Finally, for all coupling type masks, remove those types	 */
+    /* that have been declared not to participate in extraction. */
+
+    for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
+    {
+	TTMaskAndMask(&style->exts_overlapOtherTypes[s], &style->exts_activeTypes);
+	TTMaskAndMask(&style->exts_perimCapMask[s], &style->exts_activeTypes);
+
+	for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	{
+	    TTMaskAndMask(&style->exts_overlapShieldTypes[s][t], 
+			&style->exts_activeTypes);
+	    TTMaskAndMask(&style->exts_sideOverlapOtherTypes[s][t], 
+			&style->exts_activeTypes);
+	    TTMaskAndMask(&style->exts_sideCoupleOtherEdges[s][t],
+			&style->exts_activeTypes);
+	}
+    }
+
+    for (p = 0; p < DBNumPlanes; p++)
+    {
+	TTMaskAndMask(&style->exts_overlapTypes[p], &style->exts_activeTypes);
+	TTMaskAndMask(&style->exts_sideTypes[p], &style->exts_activeTypes);
+    }
+
     if ( style->exts_planeOrderStatus == noPlaneOrder ) 
     	return /* no need to check */ ;
     /* Else Check to make sure the plane order is a permutation of the
