@@ -466,7 +466,7 @@ GrTkInit(dispType)
 		visual_table[2] = j; /* GreyScale */
 	    if ((grvisual_get[j].class == 3) && (grvisual_get[j].depth == 8)
 			&& (visual_table[3] == -1))
-		visual_table[3] = j; /* Psuedocolor */
+		visual_table[3] = j; /* Pseudocolor */
 	    if ((grvisual_get[j].class == 4) && (grvisual_get[j].depth == 15)
 			&& (visual_table[4] == -1))
 		visual_table[4] = j; /* TrueColor */
@@ -841,8 +841,9 @@ keys_and_buttons:
 		idxmax = (nbytes == 0) ? 1 : nbytes;
 		for (idx = 0; idx < idxmax; idx++)
 		{
-		    if (inChar[idx] == 3)	/* Ctrl-C interrupt */
+		    if (xevent->type == KeyPress && inChar[idx] == 3)
 		    {
+			/* Ctrl-C interrupt */
 			if (SigInterruptPending)
 			    MainExit(0);		/* double Ctrl-C */
 			else
@@ -1218,6 +1219,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     GrWindowIdPtr = GrTkWindowId;
     GrWindowNamePtr = GrTkWindowName;
     GrGetCursorPosPtr = grtkGetCursorPos;
+    GrGetCursorRootPosPtr = grtkGetCursorRootPos;
 
     /* local indirections */
     grSetSPatternPtr = grtkSetSPattern;
@@ -1734,7 +1736,7 @@ GrTkIconUpdate(w, text)		/* See Blt code */
     class.res_class = "magic";
 
     XSetClassHint( grXdpy, wind, &class);
-    if (brack = index(text,'['))
+    if (brack = strchr(text,'['))
     {
      	brack--;
 	*brack = 0;
@@ -1743,7 +1745,7 @@ GrTkIconUpdate(w, text)		/* See Blt code */
      	*brack = ' ';
 	return;
     }
-    if (brack = rindex(text,' ')) text = brack+1;
+    if (brack = strrchr(text,' ')) text = brack+1;
     XSetIconName(grXdpy,wind,text);
     XStoreName(grXdpy,wind,text);
 }
