@@ -1380,7 +1380,7 @@ LefReadMacro(f, mname, oscale, importForeign)
 	    case LEF_CLASS:
 		token = LefNextToken(f, TRUE);
 		if (*token != '\n')
-		    DBPropPut(lefMacro, "LEFclass", token);
+		    DBPropPut(lefMacro, "LEFclass", StrDup((char **)NULL, token));
 		LefEndStatement(f);
 		break;
 	    case LEF_SIZE:
@@ -1420,21 +1420,25 @@ origin_error:
 		LefEndStatement(f);
 		break;
 	    case LEF_SYMMETRY:
+		strcpy(tsave, "");
 		token = LefNextToken(f, TRUE);
-		if (*token != '\n')
-		    DBPropPut(lefMacro, "LEFsymmetry", token + strlen(token) + 1);
-		LefEndStatement(f);
+		while (*token != ';')
+		{
+		    sprintf(tsave + strlen(tsave), " %s", token);
+		    token = LefNextToken(f, TRUE);
+		}
+		DBPropPut(lefMacro, "LEFsymmetry", StrDup((char **)NULL, tsave + 1));
 		break;
 	    case LEF_SOURCE:
 		token = LefNextToken(f, TRUE);
 		if (*token != '\n')
-		    DBPropPut(lefMacro, "LEFsource", token);
+		    DBPropPut(lefMacro, "LEFsource", StrDup((char **)NULL, token));
 		LefEndStatement(f);
 		break;
 	    case LEF_SITE:
 		token = LefNextToken(f, TRUE);
 		if (*token != '\n')
-		    DBPropPut(lefMacro, "LEFsite", token);
+		    DBPropPut(lefMacro, "LEFsite", StrDup((char **)NULL, token));
 		LefEndStatement(f);
 		break;
 	    case LEF_PIN:
@@ -1507,6 +1511,9 @@ origin_error:
 	}
 	else
 	{
+            char *propstr = (char *)mallocMagic(64);
+	    int reducer = DBCellFindScale(lefMacro);
+
 	    lefMacro->cd_bbox = lefBBox;
 	    lefMacro->cd_extended = lefBBox;
 	}
@@ -1853,7 +1860,7 @@ LefRead(inName, importForeign)
     }
 
     TxPrintf("Reading LEF data from file %s.\n", filename);
-    TxPrintf("This action is undoable.\n");
+    TxPrintf("This action cannot be undone.\n");
     UndoDisable();
 
     /* Initialize */
