@@ -58,6 +58,8 @@ Tcl_Interp *consoleinterp;
 
 HashTable txTclTagTable;
 
+Tcl_ChannelType inChannel;
+
 /* Forward declarations */
 
 int TerminalInputProc(ClientData, char *, int, int *);
@@ -370,12 +372,24 @@ _tcl_dispatch(ClientData clientData,
 	/* has an extension which is not ".mag", we will return the 	*/
 	/* error.							*/
 
+	/* Updated 1/20/2015:  Need to check for a '.' AFTER the last	*/
+	/* slash, so as to avoid problems with ./, ../, etc.		*/
+
 	if (idx == IDX_LOAD)
 	{
-	    char *dotptr;
-	    if ((argc >= 2) && (dotptr = strrchr(argv[1], '.')) != NULL)
-		if (strcmp(dotptr + 1, "mag"))
-		    return result;
+	    char *dotptr, *slashptr;
+	    if (argc >= 2)
+	    {
+		slashptr = strrchr(argv[1], '/');
+		if (slashptr == NULL)
+		    slashptr = argv[1];
+		else
+		    slashptr++;
+
+		if ((dotptr = strrchr(slashptr, '.')) != NULL)
+		    if (strcmp(dotptr + 1, "mag"))
+			return result;
+	    }
 	}
     }
     Tcl_ResetResult(interp);
@@ -505,6 +519,14 @@ _magic_initialize(ClientData clientData,
     if ((consoleinterp = Tcl_GetMaster(interp)) == NULL)
 	consoleinterp = interp;
 
+    // Force tkcon to send output to terminal during initialization
+    else
+    {
+	RuntimeFlags |= MAIN_TK_CONSOLE;
+    	Tcl_Eval(consoleinterp, "rename ::puts ::unused_puts\n");
+    	Tcl_Eval(consoleinterp, "rename ::tkcon_tcl_puts ::puts\n");
+    }
+
     /* Did we start in the same interpreter as we initialized? */
     if (magicinterp != interp)
     {
@@ -512,11 +534,22 @@ _magic_initialize(ClientData clientData,
 		"to handle this.\n");
 	magicinterp = interp;
     }
-    TxPrintf("Starting magic under Tcl interpreter\n");
 
     if (mainInitBeforeArgs(argc, argv) != 0) goto magicfatal;
     if (mainDoArgs(argc, argv) != 0) goto magicfatal;
 
+    // Redirect output back to the console
+    if (TxTkConsole)
+    {
+    	Tcl_Eval(consoleinterp, "rename ::puts ::tkcon_tcl_puts\n");
+    	Tcl_Eval(consoleinterp, "rename ::unused_puts ::puts\n");
+    }
+
+    /* Identify version and revision */
+
+    TxPrintf("\nMagic %s revision %s - Compiled on %s.\n", MagicVersion,
+                MagicRevision, MagicCompileTime);
+    TxPrintf("Starting magic under Tcl interpreter\n");
     if (TxTkConsole)
 	TxPrintf("Using Tk console window\n");
     else
@@ -553,6 +586,19 @@ _magic_initialize(ClientData clientData,
 
     if (strcmp(MainDisplayType, "NULL"))
 	RegisterTkCommands(interp);
+
+    /* Set up the console so that its menu option File->Exit	*/
+    /* calls magic's exit routine first.  This should not be	*/
+    /* done in console.tcl, or else it puts the console in a	*/
+    /* state where it is difficult to exit, if magic doesn't	*/
+    /* start up correctly.					*/
+
+    if (TxTkConsole)
+    {
+	Tcl_Eval(consoleinterp, "rename ::exit ::quit\n");
+	Tcl_Eval(consoleinterp, "proc ::exit args {slave eval quit}\n");
+    }
+
     return TCL_OK;
 
 magicfatal:
@@ -561,6 +607,14 @@ magicfatal:
     return TCL_ERROR;
 }
 
+/*--------------------------------------------------------------*/
+
+typedef struct FileState {
+    Tcl_Channel channel;
+    int fd;
+    int validMask;
+} FileState;
+
 /*--------------------------------------------------------------*/
 /* Post-initialization:  read in the magic startup files and	*/
 /* load any initial layout.  Note that this is not done via	*/
@@ -571,8 +625,6 @@ static int
 _magic_startup(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
-    Tcl_ChannelType *inChannel;
-
     /* Execute contents of startup files and load any initial cell */
 
     if (mainInitFinal() != 0)
@@ -596,11 +648,29 @@ _magic_startup(ClientData clientData,
     }
     else
     {
+	Tcl_Channel oldchannel;
+	Tcl_ChannelType *stdChannel;
+	FileState *fsPtr, *fsOrig;
+
 	/* Use the terminal.				  */
 	/* Replace the input proc for stdin with our own. */
 
-	inChannel = (Tcl_ChannelType *)Tcl_GetChannelType(Tcl_GetStdChannel(TCL_STDIN));
-	inChannel->inputProc = TerminalInputProc;
+	oldchannel = Tcl_GetStdChannel(TCL_STDIN);	// Get existing stdin
+	fsOrig = Tcl_GetChannelInstanceData(oldchannel);
+
+	/* Copy the structure from the old to the new channel */
+	stdChannel = (Tcl_ChannelType *)Tcl_GetChannelType(oldchannel);
+	memcpy(&inChannel, stdChannel, sizeof(Tcl_ChannelType));
+	inChannel.inputProc = TerminalInputProc;
+
+	fsPtr = (FileState *)Tcl_Alloc(sizeof(FileState));
+	fsPtr->validMask = fsOrig->validMask;
+	fsPtr->fd = fsOrig->fd;
+	fsPtr->channel = Tcl_CreateChannel(&inChannel, "stdin",
+	 	(ClientData)fsPtr, TCL_READABLE);
+
+	Tcl_SetStdChannel(fsPtr->channel, TCL_STDIN);	// Apply new stdin
+	Tcl_RegisterChannel(NULL, fsPtr->channel);
     }
 
     return TCL_OK;
@@ -964,16 +1034,6 @@ Tcl_escape(instring)
     return newstr;
 }
 
-/*--------------------------------------------------------------*/
-/* Provide input to Tcl from outside the terminal window by	*/
-/* stacking the "stdin" channel.				*/
-/*--------------------------------------------------------------*/
-
-typedef struct {
-    Tcl_Channel channel;	/* This is all the info we need */
-    int fd;
-} FileState;
-
 /*--------------------------------------------------------------*/
 
 int
@@ -983,7 +1043,7 @@ TerminalInputProc(instanceData, buf, toRead, errorCodePtr)
     int toRead;
     int *errorCodePtr;
 {
-    FileState *fsPtr = (FileState *) instanceData;
+    FileState *fsPtr = (FileState *)instanceData;
     int bytesRead, i, tlen;
     char *locbuf;
 
