@@ -30,6 +30,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #endif
 
 #include "utils/magic.h"
+#include "utils/main.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "utils/malloc.h"
@@ -412,6 +413,19 @@ MacroKey(str, verbose)
 		kmod |= ShiftMask;
 		vis += 6;
 	    }
+	    else if (*vis == '\'')
+	    {
+		// If single quotes are used to protect the integrity
+		// of the macro character, strip them now.
+
+		char *aptr = strrchr(vis, '\'');
+		if (aptr != NULL && aptr != vis)
+		{
+		   vis++;
+		   *aptr = '\0';
+		}
+		else break;	// Don't hang on an unmatched quote
+	    }
 	    else break;
 	}
 	if (!strncmp(vis, "XK_", 3)) vis += 3;
@@ -481,7 +495,11 @@ MacroKey(str, verbose)
 	return (int)str[1] - 'A' + 1;
     }
     if (warn)
-	TxError("Extended macros are unavailable with this device type.\n");
+    {
+	if (strcasecmp(MainDisplayType, "NULL") || (TxTkConsole == TRUE))
+	    TxPrintf("Extended macros are unavailable"
+			" with graphics type \"%s\".\n", MainDisplayType);
+    }
     warn = 0;
     *verbose = 0;
     return(0);
