@@ -761,9 +761,8 @@ mainInitFinal()
 	result = Tcl_EvalFile(magicinterp, rname);
 	if (result != TCL_OK)
 	{
-	    TxError("%s\n", Tcl_GetStringResult(magicinterp));
-	    TxError("System pre-startup file \"%s\" not found or unreadable!\n",
-			rname);
+	    TxError("Error parsing pre-startup file \"%s\": %s\n", rname,
+				Tcl_GetStringResult(magicinterp));
 	    Tcl_ResetResult(magicinterp);
 	}
     }
@@ -815,9 +814,8 @@ mainInitFinal()
 	result = Tcl_EvalFile(magicinterp, rname);
 	if (result != TCL_OK)
 	{
-	    TxError("%s\n", Tcl_GetStringResult(magicinterp));
-	    TxError("System startup file \"%s\" not found or unreadable!\n",
-			rname);
+	    TxError("Error parsing system startup file \"%s\": %s\n", rname,
+				Tcl_GetStringResult(magicinterp));
 	    Tcl_ResetResult(magicinterp);
 	}
     }
@@ -855,20 +853,42 @@ mainInitFinal()
 
 	if (home != NULL && (RCFileName[0] != '/'))
 	{
+	    Tcl_Channel fc;
+
 	    (void) sprintf(startupFileName, "%s/%s", home, RCFileName);
 
-	    result = Tcl_EvalFile(magicinterp, startupFileName);
-	    if (result != TCL_OK)
+	    fc = Tcl_OpenFileChannel(magicinterp, startupFileName, "r", 0);
+	    if (fc != NULL)
+	    {
+		Tcl_Close(magicinterp, fc);
+		result = Tcl_EvalFile(magicinterp, startupFileName);
+		if (result != TCL_OK)
+		{
+		    TxError("Error parsing user \"%s\": %s\n", RCFileName,
+				Tcl_GetStringResult(magicinterp));
+		    Tcl_ResetResult(magicinterp);
+		}
+	    }
+	    else
 	    {
-		Tcl_ResetResult(magicinterp);
 		/* Try the (deprecated) name ".magic" */
 		(void) sprintf(startupFileName, "%s/.magic", home);
-		result = Tcl_EvalFile(magicinterp, startupFileName);
-		if (result == TCL_OK)
+		fc = Tcl_OpenFileChannel(magicinterp, startupFileName, "r", 0);
+		if (fc != NULL)
+		{
 		    TxPrintf("Note:  Use of the file name \"~/.magic\" is deprecated."
 			"  Please change this to \"~/.magicrc\".\n");
-		else
-		    Tcl_ResetResult(magicinterp);	// Not an error
+
+		    Tcl_Close(magicinterp, fc);
+		    result = Tcl_EvalFile(magicinterp, startupFileName);
+
+		    if (result != TCL_OK)
+		    {
+			TxError("Error parsing user \".magic\": %s\n",
+				Tcl_GetStringResult(magicinterp));
+			Tcl_ResetResult(magicinterp);
+		    }
+		}
 	    }
 	}
 
@@ -877,24 +897,72 @@ mainInitFinal()
 	    /* Read in the .magicrc file from the current directory, if	*/
 	    /* different from HOME.					*/
 
-	    result = Tcl_EvalFile(magicinterp, RCFileName);
-	    if (result != TCL_OK)
+	    Tcl_Channel fc;
+
+	    fc = Tcl_OpenFileChannel(magicinterp, RCFileName, "r", 0);
+	    if (fc != NULL)
+	    {
+		Tcl_Close(magicinterp, fc);
+		result = Tcl_EvalFile(magicinterp, RCFileName);
+
+		if (result != TCL_OK)
+		{
+		    // Print error message but continue anyway
+
+		    TxError("Error parsing \"%s\": %s\n", RCFileName,
+				Tcl_GetStringResult(magicinterp));
+		    Tcl_ResetResult(magicinterp);
+		    TxPrintf("Bad local startup file \"%s\", continuing without.\n",
+					RCFileName);
+		}
+	    }
+	    else
 	    {
-		Tcl_ResetResult(magicinterp);
 		/* Try the (deprecated) name ".magic" */
-		result = Tcl_EvalFile(magicinterp, ".magic");
-		if (result == TCL_OK)
+
+		Tcl_ResetResult(magicinterp);
+		fc = Tcl_OpenFileChannel(magicinterp, ".magic", "r", 0);
+		if (fc != NULL)
+		{
+		    Tcl_Close(magicinterp, fc);
+
 		    TxPrintf("Note:  Use of the file name \".magic\" is deprecated."
 				"  Please change this to \".magicrc\".\n");
-		else
-		{
-		    Tcl_ResetResult(magicinterp);
-		    result = Tcl_EvalFile(magicinterp, "magic_setup");
+
+		    result = Tcl_EvalFile(magicinterp, ".magic");
+
 		    if (result != TCL_OK)
 		    {
-			TxPrintf("No local startup file \"%s\", continuing without.\n",
-					RCFileName);
-			Tcl_ResetResult(magicinterp);	// Still not an error
+			// Print error message but continue anyway
+
+			TxError("Error parsing local \".magic\": %s\n",
+				Tcl_GetStringResult(magicinterp));
+			Tcl_ResetResult(magicinterp);
+			TxPrintf("Bad local startup file \".magic\","
+					" continuing without.\n");
+		    }
+		}
+		else
+		{
+		    /* Try the alternative name "magic_setup" */
+
+		    Tcl_ResetResult(magicinterp);
+
+		    fc = Tcl_OpenFileChannel(magicinterp, "magic_setup", "r", 0);
+		    if (fc != NULL)
+		    {
+			Tcl_Close(magicinterp, fc);
+
+			result = Tcl_EvalFile(magicinterp, "magic_setup");
+			if (result != TCL_OK)
+			{
+			    TxError("Error parsing local \"magic_setup\": %s\n",
+					Tcl_GetStringResult(magicinterp));
+			    TxError("%s\n", Tcl_GetStringResult(magicinterp));
+			    Tcl_ResetResult(magicinterp);	// Still not an error
+			    TxPrintf("Bad local startup file \"magic_setup\","
+					" continuing without.\n");
+			}
 		    }
 		}
 	    }
